/* This file is automatically generated.
   This file selects the right generated file of `__stub_FUNCTION' macros
   based on the architecture being compiled for.  */

#include <bits/wordsize.h>

#if !defined __AARCH64EB__
# include <gnu/stubs-lp64.h>
#endif
#if defined __AARCH64EB__
# include <gnu/stubs-lp64_be.h>
#endif
